"""Core eILCD module containing valiate functionalities."""

from io import StringIO
from pathlib import Path
from typing import List, Union

from lxmlh import (
    validate_file,
)

from .config import Defaults


def validate_file_process_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Process Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Process Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_PROCESS_DATASET)


def validate_file_flow_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Flow Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Flow Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_FLOW_DATASET)


def validate_file_flow_property_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Flow Property Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Flow Property Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_FLOW_PROPERTY_DATASET)


def validate_file_unit_group_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Unit Group Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Unit Group Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_UNIT_GROUP_DATASET)


def validate_file_contact_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Contact Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Contact Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_CONTACT_DATASET)


def validate_file_source_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Source Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Source Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_SOURCE_DATASET)


def validate_file_model_dataset(
    file: Union[str, Path, StringIO]
) -> Union[None, List[str]]:
    """Validates an ILCD Model Dataset XML file against schema.
    Parameters:
    file: the str|Path path to the ILCD Source Dataset XML file or its StringIO
    representation.
    Returns ``None`` if valid or a list of error strings.
    """
    return validate_file(file, Defaults.SCHEMA_MODEL_DATASET)



"""Test cases for the __flow_property_dataset__ module."""

from pyeilcd.common import ClassificationInformation, GlobalReference
from pyeilcd.flow_property_dataset import (
    ComplianceDeclarations,
    DataEntryBy,
    FlowPropertyDataSet,
)


def test_flow_properties_information(
    flow_property_dataset: FlowPropertyDataSet,
) -> None:
    """It parses attributes correctly."""
    flowPropertiesInformation = flow_property_dataset.flowPropertiesInformation
    dataSetInformation = flowPropertiesInformation.dataSetInformation
    quantitativeReference = flowPropertiesInformation.quantitativeReference

    assert isinstance(
        dataSetInformation.classificationInformation, ClassificationInformation
    )
    assert isinstance(
        quantitativeReference.referenceToReferenceUnitGroup, GlobalReference
    )


def test_modelling_and_validation(flow_property_dataset: FlowPropertyDataSet) -> None:
    """It parses attributes correctly."""
    modellingAndValidation = flow_property_dataset.modellingAndValidation
    dataSourcesTreatmentAndRepresentativeness = (
        modellingAndValidation.dataSourcesTreatmentAndRepresentativeness
    )

    assert isinstance(
        modellingAndValidation.complianceDeclarations, ComplianceDeclarations
    )
    assert isinstance(
        dataSourcesTreatmentAndRepresentativeness.referenceToDataSources[0],
        GlobalReference,
    )


def test_administrative_information(flow_property_dataset: FlowPropertyDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = flow_property_dataset.administrativeInformation
    dataEntryBy = administrativeInformation.dataEntryBy
    publicationAndOwnership = administrativeInformation.publicationAndOwnership

    assert isinstance(dataEntryBy, DataEntryBy)
    assert isinstance(
        publicationAndOwnership.referenceToOwnershipOfDataSet, GlobalReference
    )



"""pyeilcd."""

from .config import Defaults
from .core import (
    validate_file_contact_dataset,
    validate_file_flow_dataset,
    validate_file_flow_property_dataset,
    validate_file_model_dataset,
    validate_file_process_dataset,
    validate_file_source_dataset,
    validate_file_unit_group_dataset,
)

__version__ = "7.0.14"

__all__ = (
    "__version__",
    "Defaults",
    "validate_file_contact_dataset",
    "validate_file_flow_dataset",
    "validate_file_flow_property_dataset",
    "validate_file_process_dataset",
    "validate_file_source_dataset",
    "validate_file_model_dataset",
    "validate_file_unit_group_dataset",
)



"""Test cases for the __config__ module."""

import os
from pathlib import Path

from pyeilcd.config import Defaults


def test_config_defaults() -> None:
    """It overrides defaults variables."""
    rootDir = Path(__file__).parent.parent.resolve()

    configFileDir = os.path.join(rootDir, "out", "tests")
    configFilePath = os.path.join(configFileDir, "config.ini")
    os.makedirs(configFileDir, exist_ok=True)

    schemaDir = os.path.join(rootDir, "pyilcd", "schemas")
    schemaProcessDataset = os.path.join(schemaDir, "ILCD_ProcessDataSet.xsd")
    classificationName = "ILCD"

    with open(configFilePath, "w", encoding="utf-8") as configFile:
        configFile.write("[parameters]\n")
        configFile.write(f"SCHEMA_PROCESS_DATASET={schemaProcessDataset}\n")
        configFile.write(f"[Classification]\nname={classificationName}\n")

    Defaults.config_defaults(configFilePath)

    assert Defaults.STATIC_DEFAULTS["Classification"]["name"] == classificationName

    Defaults.config_defaults("config.init")



"""Test cases for the __process_dataset__ module."""

from pyeilcd.common import ClassificationInformation, GlobalReference
from pyeilcd.process_dataset import (
    Allocation,
    CommissionerAndGoal,
    CompletenessElementaryFlows,
    Compliance,
    LocationOfOperationSupplyOrProduction,
    Name,
    ProcessDataSet,
    QuantitativeReference,
    Review,
    SubLocationOfOperationSupplyOrProduction,
    Time,
    VariableParameter,
)


def test_process_information(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    processInformation = process_dataset.processInformation
    geography = processInformation.geography
    technology = processInformation.technology

    assert isinstance(
        processInformation.quantitativeReference,
        QuantitativeReference,
    )
    assert isinstance(processInformation.time, Time)
    assert isinstance(
        geography.locationOfOperationSupplyOrProduction,
        LocationOfOperationSupplyOrProduction,
    )
    assert isinstance(
        geography.subLocationOfOperationSupplyOrProduction[0],
        SubLocationOfOperationSupplyOrProduction,
    )
    assert isinstance(technology.referenceToIncludedProcesses[0], GlobalReference)
    assert isinstance(technology.referenceToTechnologyPictogramme, GlobalReference)
    assert isinstance(
        technology.referenceToTechnologyFlowDiagrammOrPicture[0], GlobalReference
    )
    assert isinstance(
        processInformation.mathematicalRelations.variableParameter[0], VariableParameter
    )


def test_modelling_and_validation(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    modellingAndValidation = process_dataset.modellingAndValidation
    dataSourcesTreatmentAndRepresentativeness = (
        modellingAndValidation.dataSourcesTreatmentAndRepresentativeness
    )
    lciMethodAndAllocation = modellingAndValidation.lciMethodAndAllocation

    assert isinstance(
        lciMethodAndAllocation.referenceToLCAMethodDetails[0], GlobalReference
    )
    assert isinstance(
        dataSourcesTreatmentAndRepresentativeness.referenceToDataHandlingPrinciples[0],
        GlobalReference,
    )
    assert isinstance(
        dataSourcesTreatmentAndRepresentativeness.referenceToDataSource[0],
        GlobalReference,
    )
    assert isinstance(
        modellingAndValidation.completeness.completenessElementaryFlows[0],
        CompletenessElementaryFlows,
    )
    assert isinstance(
        modellingAndValidation.completeness.referenceToSupportedImpactAssessmentMethods,
        GlobalReference,
    )
    assert isinstance(modellingAndValidation.validation.reviews[0], Review)
    assert isinstance(
        modellingAndValidation.complianceDeclarations.compliances[0], Compliance
    )


def test_administrative_information(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = process_dataset.administrativeInformation
    publicationAndOwnership = administrativeInformation.publicationAndOwnership
    dataEntryBy = administrativeInformation.dataEntryBy
    dataGenerator = administrativeInformation.dataGenerator

    assert isinstance(
        administrativeInformation.commissionerAndGoal, CommissionerAndGoal
    )
    assert isinstance(
        publicationAndOwnership.referenceToRegistrationAuthority, GlobalReference
    )
    assert isinstance(
        publicationAndOwnership.referenceToOwnershipOfDataSet,
        GlobalReference,
    )
    assert isinstance(
        dataEntryBy.referenceToConvertedOriginalDataSetFrom, GlobalReference
    )
    assert isinstance(dataEntryBy.referenceToDataSetUseApproval[0], GlobalReference)
    assert isinstance(
        dataGenerator.referenceToPersonOrEntityGeneratingTheDataSet[0], GlobalReference
    )


def test_lcia_result(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    lciaResult = process_dataset.lciaResults.lciaResults[0]

    assert isinstance(lciaResult.referenceToLCIAMethodDataSets, GlobalReference)


def test_dataset_information(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    datasetInformation = process_dataset.processInformation.dataSetInformation

    assert isinstance(datasetInformation.name, Name)
    assert isinstance(
        datasetInformation.complementingProcesses.referenceToComplementingProcesses[0],
        GlobalReference,
    )
    assert isinstance(
        datasetInformation.classificationInformation,
        ClassificationInformation,
    )
    assert isinstance(
        datasetInformation.referenceToExternalDocumentation,
        GlobalReference,
    )


def test_exchange(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    exchange = process_dataset.exchanges.exchanges[0]

    assert isinstance(exchange.allocations.allocations[0], Allocation)
    assert isinstance(
        exchange.referencesToDataSource.referenceToDataSources[0], GlobalReference
    )
    assert isinstance(exchange.referenceToFlowDataSet, GlobalReference)



"""Test cases for the __common__ module."""

from pyeilcd.common import (
    Category,
    Class,
    Classification,
    DataQualityIndicator,
    GlobalReference,
    Method,
)
from pyeilcd.flow_dataset import FlowDataSet
from pyeilcd.process_dataset import ProcessDataSet
from pyeilcd.unit_group_dataset import UnitGroupDataSet


def test_classification_information(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    dataSetInformation = process_dataset.processInformation.dataSetInformation
    classificationInformation = dataSetInformation.classificationInformation

    assert isinstance(
        classificationInformation.classifications[0].classesList[0], Class
    )


def test_review(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    modellingAndValidation = process_dataset.modellingAndValidation
    review = modellingAndValidation.validation.reviews[0]

    assert isinstance(review.scope.method[0], Method)
    assert isinstance(
        review.dataQualityIndicators.dataQualityIndicators[0], DataQualityIndicator
    )
    assert isinstance(review.referenceToCompleteReviewReport, GlobalReference)
    assert isinstance(review.referenceToNameOfReviewerAndInstitution, GlobalReference)


def test_compliance(unit_group_dataset: UnitGroupDataSet) -> None:
    """It parses attributes correctly."""
    modellingAndValidation = unit_group_dataset.modellingAndValidation
    compliance = modellingAndValidation.complianceDeclarations.compliances[0]

    assert isinstance(compliance.referenceToComplianceSystem, GlobalReference)


def test_commissioner_and_goal(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = process_dataset.administrativeInformation
    commissionerAndGoal = administrativeInformation.commissionerAndGoal

    assert isinstance(commissionerAndGoal.referenceToCommissioner[0], GlobalReference)


def test_data_entry_by(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = process_dataset.administrativeInformation
    dataEntryBy = administrativeInformation.dataEntryBy

    assert isinstance(dataEntryBy.referenceToDataSetFormat[0], GlobalReference)
    assert isinstance(
        dataEntryBy.referenceToPersonOrEntityEnteringTheData, GlobalReference
    )


def test_publication_and_ownership(process_dataset: ProcessDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = process_dataset.administrativeInformation
    publicationAndOwnership = administrativeInformation.publicationAndOwnership

    assert isinstance(
        publicationAndOwnership.referenceToPrecedingDataSetVersion[0], GlobalReference
    )
    assert isinstance(
        publicationAndOwnership.referenceToUnchangedRepublication, GlobalReference
    )
    assert isinstance(
        publicationAndOwnership.referenceToEntitiesWithExclusiveAccess[0],
        GlobalReference,
    )


def test_flow_category_information(flow_dataset: FlowDataSet) -> None:
    """It parses attributes correctly."""
    dataSetInformation = flow_dataset.flowInformation.dataSetInformation
    flowCategoryInformation = dataSetInformation.classificationInformation

    assert isinstance(
        flowCategoryInformation.elementaryFlowCategorization[0].categoryList[0],
        Category,
    )
    assert isinstance(flowCategoryInformation.classifications[0], Classification)



"""Defaults configuration."""

import configparser
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Callable, ClassVar, Dict

from lxml import etree


@dataclass
class Defaults:
    """Stores default values for ILCD attributes used when no value exists."""

    SCHEMA_DIR: ClassVar[str] = os.path.join(Path(__file__).parent.resolve(), "schemas")
    SCHEMA_PROCESS_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_ProcessDataSet.xsd"
    )
    SCHEMA_FLOW_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_FlowDataSet.xsd"
    )
    SCHEMA_FLOW_PROPERTY_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_FlowPropertyDataSet.xsd"
    )
    SCHEMA_UNIT_GROUP_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_UnitGroupDataSet.xsd"
    )
    SCHEMA_CONTACT_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_ContactDataSet.xsd"
    )
    SCHEMA_SOURCE_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_SourceDataSet.xsd"
    )
    SCHEMA_MODEL_DATASET: ClassVar[str] = os.path.join(
        SCHEMA_DIR, "ILCD_LifeCycleModelDataSet.xsd"
    )

    DYNAMIC_DEFAULTS: ClassVar[
        Dict[str, Dict[str, Callable[[etree.ElementBase], str]]]
    ] = {}
    STATIC_DEFAULTS: ClassVar[Dict[str, Dict[str, str]]] = {
        "Classification": {
            "name": "ILCD",
        },
        "FlowCategorization": {
            "name": "ILCD",
        },
        "ProcessDataset": {
            "metaDataOnly": "false",
        },
    }

    @classmethod
    def config_defaults(cls, config_file: str) -> None:
        """Fully/ partially overrides defaults.
        Parameters:
        config_file: path for config file.
        """
        config = configparser.ConfigParser()
        config.optionxform = lambda optionstr: optionstr
        config.read(config_file)

        if config.has_section("parameters"):
            for key, value in dict(config["parameters"]).items():
                setattr(cls, key, value)

        staticDefaults = {
            name: dict(section)
            for name, section in config.items()
            if name not in ["parameters"]
        }
        cls.static_defaults = staticDefaults



"""Test cases for the __contact_dataset__ module."""

from pyeilcd.common import ClassificationInformation, GlobalReference
from pyeilcd.source_dataset import DataEntryBy, ReferenceToDigitalFile, SourceDataSet


def test_contact_information(source_dataset: SourceDataSet) -> None:
    """It parses attributes correctly."""
    sourceInformation = source_dataset.sourceInformation
    dataSetInformation = sourceInformation.dataSetInformation

    assert isinstance(
        dataSetInformation.classificationInformation, ClassificationInformation
    )
    assert isinstance(
        dataSetInformation.referenceToDigitalFiles[0], ReferenceToDigitalFile
    )
    assert isinstance(dataSetInformation.referenceToContact[0], GlobalReference)
    assert isinstance(dataSetInformation.referenceToLogo, GlobalReference)


def test_administrative_information(source_dataset: SourceDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = source_dataset.administrativeInformation

    assert isinstance(administrativeInformation.dataEntryBy, DataEntryBy)
    assert isinstance(
        administrativeInformation.publicationAndOwnership.referenceToOwnershipOfDataSet,
        GlobalReference,
    )



"""Test cases for the __unit_group_dataset__ module."""

from pyeilcd.common import ClassificationInformation, GlobalReference
from pyeilcd.unit_group_dataset import (
    DataEntryBy,
    QuantitativeReference,
    Unit,
    UnitGroupDataSet,
)


def test_unit_group_information(unit_group_dataset: UnitGroupDataSet) -> None:
    """It parses attributes correctly."""
    unitGroupInformation = unit_group_dataset.unitGroupInformation
    dataSetInformation = unitGroupInformation.dataSetInformation
    quantitativeReference = unitGroupInformation.quantitativeReference

    assert isinstance(
        dataSetInformation.classificationInformation, ClassificationInformation
    )
    assert isinstance(quantitativeReference, QuantitativeReference)


def test_administrative_information(unit_group_dataset: UnitGroupDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = unit_group_dataset.administrativeInformation

    assert isinstance(administrativeInformation.dataEntryBy, DataEntryBy)
    assert isinstance(
        administrativeInformation.publicationAndOwnership.referenceToOwnershipOfDataSet,
        GlobalReference,
    )


def test_units(unit_group_dataset: UnitGroupDataSet) -> None:
    """It parses attributes correctly."""
    units = unit_group_dataset.units

    assert isinstance(units.units[0], Unit)



"""Test cases for the __core__ module."""

import os
import tempfile
from io import StringIO
from pathlib import Path
from typing import Callable, List, Tuple, Union

from lxml import etree

from pyeilcd import (
    ContactDataSet,
    FlowDataSet,
    FlowPropertyDataSet,
    ProcessDataSet,
    SourceDataSet,
    UnitGroupDataSet,
    parse_directory_contact_dataset,
    parse_directory_flow_dataset,
    parse_directory_flow_property_dataset,
    parse_directory_process_dataset,
    parse_directory_source_dataset,
    parse_directory_unit_group_dataset,
    parse_file_process_dataset,
    parse_zip_file_contact_dataset,
    parse_zip_file_flow_dataset,
    parse_zip_file_flow_property_dataset,
    parse_zip_file_process_dataset,
    parse_zip_file_source_dataset,
    parse_zip_file_unit_group_dataset,
    save_ilcd_file,
    validate_directory_contact_dataset,
    validate_directory_flow_dataset,
    validate_directory_flow_property_dataset,
    validate_directory_process_dataset,
    validate_directory_source_dataset,
    validate_directory_unit_group_dataset,
    validate_file_contact_dataset,
    validate_file_flow_dataset,
    validate_file_flow_property_dataset,
    validate_file_process_dataset,
    validate_file_source_dataset,
    validate_file_unit_group_dataset,
    validate_zip_file_contact_dataset,
    validate_zip_file_flow_dataset,
    validate_zip_file_flow_property_dataset,
    validate_zip_file_process_dataset,
    validate_zip_file_source_dataset,
    validate_zip_file_unit_group_dataset,
)

from . import (
    FILE_CONTACT_DATASET,
    FILE_FLOW_DATASET,
    FILE_FLOW_PROPERTY_DATASET,
    FILE_PROCESS_DATASET,
    FILE_SOURCE_DATASET,
    FILE_UNIT_GROUP_DATASET,
)


def test_validate_file_process_dataset_success() -> None:
    """It validates file successfully."""
    assert validate_file_process_dataset(FILE_PROCESS_DATASET) is None


def test_validate_file_flow_dataset_success() -> None:
    """It validates file successfully."""
    assert validate_file_flow_dataset(FILE_FLOW_DATASET) is None


def test_validate_file_flow_property_dataset_success() -> None:
    """It validates file successfully."""
    assert validate_file_flow_property_dataset(FILE_FLOW_PROPERTY_DATASET) is None


def test_validate_file_unit_group_dataset_success() -> None:
    """It validates file successfully."""
    assert validate_file_unit_group_dataset(FILE_UNIT_GROUP_DATASET) is None


def test_validate_file_contact_dataset_success() -> None:
    """It validates file successfully."""
    assert validate_file_contact_dataset(FILE_CONTACT_DATASET) is None


def test_validate_file_source_dataset_success() -> None:
    """It validates file successfully."""
    assert validate_file_source_dataset(FILE_SOURCE_DATASET) is None


def _validate_file_fail(
    validator: Callable[[Union[str, Path, StringIO]], Union[None, List[str]]]
) -> None:
    xml = StringIO("<ilcd></ilcd>")
    errorExpected = (
        "<string>:1:0:ERROR:SCHEMASV:SCHEMAV_CVC_ELT_1: Element 'ilcd': "
        "No matching global declaration available for the validation root."
    )
    errorActual = validator(xml)
    assert errorActual is not None
    assert str(errorActual[0]) == errorExpected


def test_validate_file_process_dataset_fail() -> None:
    """It validates file successfully."""
    _validate_file_fail(validate_file_process_dataset)


def test_validate_file_flow_dataset_fail() -> None:
    """It validates file successfully."""
    _validate_file_fail(validate_file_flow_dataset)


def test_validate_file_flow_property_dataset_fail() -> None:
    """It validates file successfully."""
    _validate_file_fail(validate_file_flow_property_dataset)


def test_validate_file_unit_group_dataset_fail() -> None:
    """It validates file successfully."""
    _validate_file_fail(validate_file_unit_group_dataset)


def test_validate_file_contact_dataset_fail() -> None:
    """It validates file successfully."""
    _validate_file_fail(validate_file_contact_dataset)


def test_validate_file_source_dataset_fail() -> None:
    """It validates file successfully."""
    _validate_file_fail(validate_file_source_dataset)


def test_save_ilcd_file() -> None:
    """It saves read file correctly."""
    inputPath = "data/process/sample_process.xml"
    processDataset = parse_file_process_dataset(inputPath)
    outputPath = os.path.join(tempfile.gettempdir(), os.urandom(24).hex())
    save_ilcd_file(processDataset, outputPath, fill_defaults=False)

    with open(inputPath, encoding="utf-8") as inputFile:
        with open(outputPath, encoding="utf-8") as outputFile:
            mapping = {ord(c): "" for c in [" ", "\t", "\n"]}
            translatedOutput = outputFile.read().translate(mapping)
            translatedInput = inputFile.read().translate(mapping)
            assert translatedOutput == translatedInput


def test_save_file_defaults() -> None:
    """It saves read file correctly."""
    inputPath = FILE_PROCESS_DATASET
    expectedOutputPath = FILE_PROCESS_DATASET
    processDataset = parse_file_process_dataset(inputPath)
    outputPath = os.path.join(tempfile.gettempdir(), os.urandom(24).hex())
    save_ilcd_file(processDataset, outputPath, fill_defaults=True)

    with open(expectedOutputPath, encoding="utf-8") as inputFile:
        with open(outputPath, encoding="utf-8") as outputFile:
            mapping = {ord(c): "" for c in [" ", "\t", "\n"]}
            translatedOutput = outputFile.read().translate(mapping)
            translatedInput = inputFile.read().translate(mapping)
            assert translatedOutput == translatedInput


def _parse_directory(
    dataset_name: str,
    parser: Callable[
        [Union[str, Path], Union[List[str], None]], List[Tuple[Path, etree.ElementBase]]
    ],
) -> None:
    """It reads all files successfully."""
    dirPath = os.path.join(Path(__file__).parents[1], "data", dataset_name)
    files = [os.path.join(dirPath, f"sample_{dataset_name}.xml")]
    result = parser(dirPath)

    assert len(result) == len(files)
    assert result[0][0] == Path(files[0])


def test_parse_directory_process_dataset() -> None:
    "It parses directory successfully."
    _parse_directory("process", parse_directory_process_dataset)


def test_parse_directory_flow_dataset() -> None:
    "It parses directory successfully."
    _parse_directory("flow", parse_directory_flow_dataset)


def test_parse_directory_flow_property_dataset() -> None:
    "It parses directory successfully."
    _parse_directory("flow_property", parse_directory_flow_property_dataset)


def test_parse_directory_unit_group_dataset() -> None:
    "It parses directory successfully."
    _parse_directory("unit_group", parse_directory_unit_group_dataset)


def test_parse_directory_contact_dataset() -> None:
    "It parses directory successfully."
    _parse_directory("contact", parse_directory_contact_dataset)


def test_parse_directory_source_dataset() -> None:
    "It parses directory successfully."
    _parse_directory("source", parse_directory_source_dataset)


def _validate_directory(
    dataset_name: str,
    validator: Callable[
        [Union[str, Path], Union[List[str], None]], List[Tuple[Path, etree.ElementBase]]
    ],
) -> None:
    """It reads all files successfully."""
    dirPath = os.path.join(Path(__file__).parents[1], "data", dataset_name)
    files = [os.path.join(dirPath, f"sample_{dataset_name}.xml")]
    result = validator(dirPath)

    assert len(result) == len(files)
    assert result[0][0] == Path(files[0])
    assert result[0][1] is None


def test_validate_directory_process_dataset() -> None:
    "It validates directory successfully."
    _validate_directory("process", validate_directory_process_dataset)


def test_validate_directory_flow_dataset() -> None:
    "It validates directory successfully."
    _validate_directory("flow", validate_directory_flow_dataset)


def test_validate_directory_flow_property_dataset() -> None:
    "It validates directory successfully."
    _validate_directory("flow_property", validate_directory_flow_property_dataset)


def test_validate_directory_unit_group_dataset() -> None:
    "It validates directory successfully."
    _validate_directory("unit_group", validate_directory_unit_group_dataset)


def test_validate_directory_contact_dataset() -> None:
    "It validates directory successfully."
    _validate_directory("contact", validate_directory_contact_dataset)


def test_validate_directory_source_dataset() -> None:
    "It validates directory successfully."
    _validate_directory("source", validate_directory_source_dataset)


def test_validate_zip_file_process_dataset(process_dataset_zip) -> None:
    """It validates zip file successfully."""
    assert validate_zip_file_process_dataset(process_dataset_zip)[0][1] is None


def test_validate_zip_file_flow_dataset(flow_dataset_zip) -> None:
    """It validates zip file successfully."""
    assert validate_zip_file_flow_dataset(flow_dataset_zip)[0][1] is None


def test_validate_zip_file_flow_property_dataset(flow_property_dataset_zip) -> None:
    """It validates zip file successfully."""
    assert (
        validate_zip_file_flow_property_dataset(flow_property_dataset_zip)[0][1] is None
    )


def test_validate_zip_file_unit_group_dataset(unit_group_dataset_zip) -> None:
    """It validates zip file successfully."""
    assert validate_zip_file_unit_group_dataset(unit_group_dataset_zip)[0][1] is None


def test_validate_zip_file_contact_dataset(contact_dataset_zip) -> None:
    """It validates zip file successfully."""
    assert validate_zip_file_contact_dataset(contact_dataset_zip)[0][1] is None


def test_validate_zip_file_source_dataset(source_dataset_zip) -> None:
    """It validates zip file successfully."""
    assert validate_zip_file_source_dataset(source_dataset_zip)[0][1] is None


def _parse_zip_file(
    file_path: str,
    parser: Callable[
        [Union[str, Path], Union[List[str], None]], List[Tuple[Path, etree.ElementBase]]
    ],
    root_class: etree.ElementBase,
) -> None:
    """It reads zip file successfully."""
    results = parser(file_path)

    assert len(results) == 1
    assert isinstance(results[0][1], root_class)


def test_parse_zip_file_process_dataset(process_dataset_zip) -> None:
    """It reads zip file successfully."""
    _parse_zip_file(process_dataset_zip, parse_zip_file_process_dataset, ProcessDataSet)


def test_parse_zip_file_flow_dataset(flow_dataset_zip) -> None:
    """It reads zip file successfully."""
    _parse_zip_file(flow_dataset_zip, parse_zip_file_flow_dataset, FlowDataSet)


def test_parse_zip_file_flow_property_dataset(flow_property_dataset_zip) -> None:
    """It reads zip file successfully."""
    _parse_zip_file(
        flow_property_dataset_zip,
        parse_zip_file_flow_property_dataset,
        FlowPropertyDataSet,
    )


def test_parse_zip_file_unit_group_dataset(unit_group_dataset_zip) -> None:
    """It reads zip file successfully."""
    _parse_zip_file(
        unit_group_dataset_zip, parse_zip_file_unit_group_dataset, UnitGroupDataSet
    )


def test_parse_zip_file_contact_dataset(contact_dataset_zip) -> None:
    """It reads zip file successfully."""
    _parse_zip_file(contact_dataset_zip, parse_zip_file_contact_dataset, ContactDataSet)


def test_parse_zip_file_source_dataset(source_dataset_zip) -> None:
    """It reads zip file successfully."""
    _parse_zip_file(source_dataset_zip, parse_zip_file_source_dataset, SourceDataSet)



"""Test cases for the __contact_dataset__ module."""

from pyeilcd.common import ClassificationInformation, GlobalReference
from pyeilcd.contact_dataset import ContactDataSet, DataEntryBy


def test_contact_information(contact_dataset: ContactDataSet) -> None:
    """It parses attributes correctly."""
    unitGroupInformation = contact_dataset.contactInformation
    dataSetInformation = unitGroupInformation.dataSetInformation

    assert isinstance(
        dataSetInformation.classificationInformation, ClassificationInformation
    )
    assert isinstance(dataSetInformation.referenceToContact[0], GlobalReference)
    assert isinstance(dataSetInformation.referenceToLogo, GlobalReference)
    assert dataSetInformation.UUID == "00000000-0000-0000-0000-000000000000"


def test_administrative_information(contact_dataset: ContactDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = contact_dataset.administrativeInformation

    assert isinstance(administrativeInformation.dataEntryBy, DataEntryBy)
    assert isinstance(
        administrativeInformation.publicationAndOwnership.referenceToOwnershipOfDataSet,
        GlobalReference,
    )



"""Test cases for the __flow_dataset__ module."""

from pyeilcd.common import GlobalReference
from pyeilcd.flow_dataset import (
    ComplianceDeclarations,
    DataEntryBy,
    FlowCategoryInformation,
    FlowDataSet,
    Geography,
    LCIMethod,
    Name,
    QuantitativeReference,
)


def test_flow_information(flow_dataset: FlowDataSet) -> None:
    """It parses attributes correctly."""
    flowInformation = flow_dataset.flowInformation
    datasetInformation = flowInformation.dataSetInformation
    quantitativeReference = flowInformation.quantitativeReference
    technology = flowInformation.technology
    geography = flowInformation.geography

    assert isinstance(datasetInformation.name, Name)
    assert isinstance(
        datasetInformation.classificationInformation, FlowCategoryInformation
    )
    assert isinstance(quantitativeReference, QuantitativeReference)
    assert isinstance(geography, Geography)
    assert isinstance(technology.referenceToTechnicalSpecification[0], GlobalReference)


def test_modelling_and_validation(flow_dataset: FlowDataSet) -> None:
    """It parses attributes correctly."""
    modellingAndValidation = flow_dataset.modellingAndValidation

    assert isinstance(modellingAndValidation.lciMethod, LCIMethod)
    assert isinstance(
        modellingAndValidation.complianceDeclarations, ComplianceDeclarations
    )


def test_administrative_information(flow_dataset: FlowDataSet) -> None:
    """It parses attributes correctly."""
    administrativeInformation = flow_dataset.administrativeInformation

    assert isinstance(administrativeInformation.dataEntryBy, DataEntryBy)
    assert isinstance(
        administrativeInformation.publicationAndOwnership.referenceToOwnershipOfDataSet,
        GlobalReference,
    )


def test_flow_properties(flow_dataset: FlowDataSet) -> None:
    """It parses attributes correctly."""
    flowProperty = flow_dataset.flowProperties.flowProperties[0]

    assert isinstance(flowProperty.referenceToFlowPropertyDataSet, GlobalReference)
